@@ -18,5 +18,4 @@ ax.plot(np.random.rand(20), '-o', ms=20, lw=2, alpha=0.7, mfc='orange')
 ax.grid()
 fig.figimage(im, 10, 10)
 
-fig.savefig('watermarked', transparent=True)
-
+plt.show()
