@@ -1,8 +1,9 @@
 from matplotlib.dates import strpdate2num
-from matplotlib.mlab import load
+#from matplotlib.mlab import load
+import numpy as np
 from pylab import figure, show
 
-dates, closes = load(
+dates, closes = np.loadtxt(
     '../data/msft.csv', delimiter=',',
     converters={0:strpdate2num('%d-%b-%y')},
     skiprows=1, usecols=(0,2), unpack=True)
