@@ -1,9 +1,9 @@
 from matplotlib.widgets import MultiCursor
-from pylab import figure, show, nx
+from pylab import figure, show, pi, arange, sin
 
-t = nx.arange(0.0, 2.0, 0.01)
-s1 = nx.sin(2*nx.pi*t)
-s2 = nx.sin(4*nx.pi*t)
+t = arange(0.0, 2.0, 0.01)
+s1 = sin(2*pi*t)
+s2 = sin(4*pi*t)
 fig = figure()
 ax1 = fig.add_subplot(211)
 ax1.plot(t, s1)
