@@ -3,12 +3,16 @@ from __future__ import division
 import os, sys
 def fn_name(): return sys._getframe(1).f_code.co_name
 
-import gobject
-import gtk; gdk = gtk.gdk
-import pango
-pygtk_version_required = (2,2,0)
+try:
+    import gobject
+    import gtk; gdk = gtk.gdk
+    import pango
+except ImportError:
+    raise ImportError("Gtk* backend requires pygtk to be installed.")
+
+pygtk_version_required = (2,4,0)
 if gtk.pygtk_version < pygtk_version_required:
-    raise SystemExit ("PyGTK %d.%d.%d is installed\n"
+    raise ImportError ("PyGTK %d.%d.%d is installed\n"
                       "PyGTK %d.%d.%d or later is required"
                       % (gtk.pygtk_version + pygtk_version_required))
 del pygtk_version_required
@@ -56,7 +60,7 @@ def draw_if_interactive():
     if matplotlib.is_interactive():
         figManager =  Gcf.get_active()
         if figManager is not None:
-            figManager.canvas.draw()
+            figManager.canvas.draw_idle()
 
 
 def show(mainloop=True):
@@ -143,6 +147,7 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
                   gdk.EXPOSURE_MASK       |
                   gdk.KEY_PRESS_MASK      |
                   gdk.KEY_RELEASE_MASK    |
+                  gdk.ENTER_NOTIFY_MASK   |
                   gdk.LEAVE_NOTIFY_MASK   |
                   gdk.POINTER_MOTION_MASK |
                   gdk.POINTER_MOTION_HINT_MASK)
@@ -152,13 +157,13 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
         FigureCanvasBase.__init__(self, figure)
         gtk.DrawingArea.__init__(self)
 
-        self._idleID        = 0
+        self._idle_draw_id  = 0
         self._need_redraw   = True
         self._pixmap_width  = -1
         self._pixmap_height = -1
         self._lastCursor    = None
 
-        self.connect('scroll_event',   self.scroll_event)
+        self.connect('scroll_event',         self.scroll_event)
         self.connect('button_press_event',   self.button_press_event)
         self.connect('button_release_event', self.button_release_event)
         self.connect('configure_event',      self.configure_event)
@@ -166,6 +171,8 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
         self.connect('key_press_event',      self.key_press_event)
         self.connect('key_release_event',    self.key_release_event)
         self.connect('motion_notify_event',  self.motion_notify_event)
+        self.connect('leave_notify_event',   self.leave_notify_event)
+        self.connect('enter_notify_event',   self.enter_notify_event)
 
         self.set_events(self.__class__.event_mask)
 
@@ -173,7 +180,13 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
         self.set_flags(gtk.CAN_FOCUS)
         self._renderer_init()
 
-        gobject.idle_add(self.idle_event)
+        self._idle_event_id = gobject.idle_add(self.idle_event)
+
+    def destroy(self):
+        #gtk.DrawingArea.destroy(self)
+        gobject.source_remove(self._idle_event_id)
+        if self._idle_draw_id != 0:
+            gobject.source_remove(self._idle_draw_id)
 
     def scroll_event(self, widget, event):
         if _debug: print 'FigureCanvasGTK.%s' % fn_name()
@@ -184,7 +197,7 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
             step = 1
         else:
             step = -1
-        FigureCanvasBase.scroll_event(self, x, y, step)
+        FigureCanvasBase.scroll_event(self, x, y, step, guiEvent=event)
         return False  # finish event propagation?
 
     def button_press_event(self, widget, event):
@@ -192,7 +205,7 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
         x = event.x
         # flipy so y=0 is bottom of canvas
         y = self.allocation.height - event.y
-        FigureCanvasBase.button_press_event(self, x, y, event.button)
+        FigureCanvasBase.button_press_event(self, x, y, event.button, guiEvent=event)
         return False  # finish event propagation?
 
     def button_release_event(self, widget, event):
@@ -200,21 +213,21 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
         x = event.x
         # flipy so y=0 is bottom of canvas
         y = self.allocation.height - event.y
-        FigureCanvasBase.button_release_event(self, x, y, event.button)
+        FigureCanvasBase.button_release_event(self, x, y, event.button, guiEvent=event)
         return False  # finish event propagation?
 
     def key_press_event(self, widget, event):
         if _debug: print 'FigureCanvasGTK.%s' % fn_name()
         key = self._get_key(event)
         if _debug: print "hit", key
-        FigureCanvasBase.key_press_event(self, key)
+        FigureCanvasBase.key_press_event(self, key, guiEvent=event)
         return False  # finish event propagation?
 
     def key_release_event(self, widget, event):
         if _debug: print 'FigureCanvasGTK.%s' % fn_name()
         key = self._get_key(event)
         if _debug: print "release", key
-        FigureCanvasBase.key_release_event(self, key)
+        FigureCanvasBase.key_release_event(self, key, guiEvent=event)
         return False  # finish event propagation?
 
     def motion_notify_event(self, widget, event):
@@ -226,11 +239,17 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
 
         # flipy so y=0 is bottom of canvas
         y = self.allocation.height - y
-        FigureCanvasBase.motion_notify_event(self, x, y)
+        FigureCanvasBase.motion_notify_event(self, x, y, guiEvent=event)
         return False  # finish event propagation?
 
+    def leave_notify_event(self, widget, event):
+        FigureCanvasBase.leave_notify_event(self, event)
+
+    def enter_notify_event(self, widget, event):
+        FigureCanvasBase.enter_notify_event(self, event)
+
     def _get_key(self, event):
-        if self.keyvald.has_key(event.keyval):
+        if event.keyval in self.keyvald:
             key = self.keyvald[event.keyval]
         elif event.keyval <256:
             key = chr(event.keyval)
@@ -272,10 +291,10 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
     def draw_idle(self):
         def idle_draw(*args):
             self.draw()
-            self._idleID = 0
+            self._idle_draw_id = 0
             return False
-        if self._idleID == 0:
-            self._idleID = gobject.idle_add(idle_draw)
+        if self._idle_draw_id == 0:
+            self._idle_draw_id = gobject.idle_add(idle_draw)
 
 
     def _renderer_init(self):
@@ -420,7 +439,16 @@ class FigureManagerGTK(FigureManagerBase):
 
         self.window = gtk.Window()
         self.window.set_title("Figure %d" % num)
-
+        if (window_icon):
+            try:
+                self.window.set_icon_from_file(window_icon)
+            except:
+                # some versions of gtk throw a glib.GError but not
+                # all, so I am not sure how to catch it.  I am unhappy
+                # diong a blanket catch here, but an not sure what a
+                # better way is - JDH
+                verbose.report('Could not load matplotlib icon: %s' % sys.exc_info()[1])
+                
         self.vbox = gtk.VBox()
         self.window.add(self.vbox)
         self.vbox.show()
@@ -462,14 +490,18 @@ class FigureManagerGTK(FigureManagerBase):
 
     def destroy(self, *args):
         if _debug: print 'FigureManagerGTK.%s' % fn_name()
+        self.vbox.destroy()
         self.window.destroy()
+        self.canvas.destroy()
+        if self.toolbar:
+            self.toolbar.destroy()
+        self.__dict__.clear()
 
         if Gcf.get_num_fig_managers()==0 and \
                not matplotlib.is_interactive() and \
                gtk.main_level() >= 1:
             gtk.main_quit()
 
-
     def show(self):
         # show the figure window
         self.window.show()
@@ -523,14 +555,10 @@ class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
         self.win = window
         gtk.Toolbar.__init__(self)
         NavigationToolbar2.__init__(self, canvas)
-        self._idleId = 0
-
-        def destroy(*args):
-            self.fileselect.destroy()
-        self.connect("destroy", destroy)
+        self._idle_draw_id = 0
 
     def set_message(self, s):
-        if self._idleId == 0:
+        if self._idle_draw_id == 0:
             self.message.set_label(s)
 
     def set_cursor(self, cursor):
@@ -572,54 +600,20 @@ class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
             axrect = l,b,w,h
             self._imageBack = axrect, drawable.get_image(*axrect)
             drawable.draw_rectangle(gc, False, *rect)
-            self._idleId = 0
+            self._idle_draw_id = 0
         else:
             def idle_draw(*args):
-
                 drawable.draw_image(gc, imageBack, 0, 0, *lastrect)
                 drawable.draw_rectangle(gc, False, *rect)
-                self._idleId = 0
+                self._idle_draw_id = 0
                 return False
-            if self._idleId == 0:
-                self._idleId = gobject.idle_add(idle_draw)
+            if self._idle_draw_id == 0:
+                self._idle_draw_id = gobject.idle_add(idle_draw)
 
 
     def _init_toolbar(self):
         self.set_style(gtk.TOOLBAR_ICONS)
-
-        if gtk.pygtk_version >= (2,4,0):
-            self._init_toolbar2_4()
-        else:
-            self._init_toolbar2_2()
-
-
-    def _init_toolbar2_2(self):
-        basedir = os.path.join(matplotlib.rcParams['datapath'],'images')
-
-        for text, tooltip_text, image_file, callback in self.toolitems:
-            if text is None:
-                 self.append_space()
-                 continue
-
-            fname = os.path.join(basedir, image_file)
-            image = gtk.Image()
-            image.set_from_file(fname)
-            w = self.append_item(text,
-                                 tooltip_text,
-                                 'Private',
-                                 image,
-                                 getattr(self, callback)
-                                 )
-
-        self.append_space()
-
-        self.message = gtk.Label()
-        self.append_widget(self.message, None, None)
-        self.message.show()
-
-        self.fileselect = FileSelection(title='Save the figure',
-                                        parent=self.win,)
-
+        self._init_toolbar2_4()
 
 
     def _init_toolbar2_4(self):
@@ -652,15 +646,15 @@ class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
 
         self.show_all()
 
-        self.fileselect = FileChooserDialog(
+    def get_filechooser(self):
+        return FileChooserDialog(
             title='Save the figure',
             parent=self.win,
             filetypes=self.canvas.get_supported_filetypes(),
             default_filetype=self.canvas.get_default_filetype())
 
-
     def save_figure(self, button):
-        fname, format = self.fileselect.get_filename_from_user()
+        fname, format = self.get_filechooser().get_filename_from_user()
         if fname:
             try:
                 self.canvas.print_figure(fname, format=format)
@@ -678,6 +672,12 @@ class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
 
 
         window = gtk.Window()
+        if (window_icon):
+            try: window.set_icon_from_file(window_icon)
+            except:
+                # we presumably already logged a message on the
+                # failure of the main plot, don't keep reporting
+                pass
         window.set_title("Subplot Configuration Tool")
         window.set_default_size(w, h)
         vbox = gtk.VBox()
@@ -747,27 +747,16 @@ class NavigationToolbar(gtk.Toolbar):
 
         self.set_style(gtk.TOOLBAR_ICONS)
 
-        if gtk.pygtk_version >= (2,4,0):
-            self._create_toolitems_2_4()
-            self.update = self._update_2_4
-            self.fileselect = FileChooserDialog(
-                title='Save the figure',
-                parent=self.win,
-                filetypes=self.canvas.get_supported_filetypes(),
-                default_filetype=self.canvas.get_default_filetype())
-        else:
-            self._create_toolitems_2_2()
-            self.update = self._update_2_2
-            self.fileselect = FileSelection(title='Save the figure',
-                                            parent=self.win)
+        self._create_toolitems_2_4()
+        self.update = self._update_2_4
+        self.fileselect = FileChooserDialog(
+            title='Save the figure',
+            parent=self.win,
+            filetypes=self.canvas.get_supported_filetypes(),
+            default_filetype=self.canvas.get_default_filetype())
         self.show_all()
         self.update()
 
-        def destroy(*args):
-            self.fileselect.destroy()
-            del self.fileselect
-        self.connect("destroy", destroy)
-
     def _create_toolitems_2_4(self):
         # use the GTK+ 2.4 GtkToolbar API
         iconSize = gtk.ICON_SIZE_SMALL_TOOLBAR
@@ -844,46 +833,6 @@ class NavigationToolbar(gtk.Toolbar):
         self.set_active(range(len(self._axes)))
 
 
-    def _create_toolitems_2_2(self):
-        # use the GTK+ 2.2 (and lower) GtkToolbar API
-        iconSize = gtk.ICON_SIZE_SMALL_TOOLBAR
-
-        for text, tooltip_text, image_num, callback, callback_arg, scroll \
-                in self.toolitems:
-            if text is None:
-                self.append_space()
-                continue
-            image = gtk.Image()
-            image.set_from_stock(image_num, iconSize)
-            item = self.append_item(text, tooltip_text, 'Private', image,
-                                    getattr(self, callback), callback_arg)
-            if scroll:
-                item.connect("scroll_event", getattr(self, callback))
-
-        self.omenu = gtk.OptionMenu()
-        self.omenu.set_border_width(3)
-        self.insert_widget(
-            self.omenu,
-            'Select axes that controls affect',
-            'Private', 0)
-
-
-    def _update_2_2(self):
-        # for GTK+ 2.2 and lower
-        # called by __init__() and FigureManagerGTK
-
-        self._axes = self.canvas.figure.axes
-
-        if len(self._axes) >= 2:
-            # set up the axis menu
-            self.omenu.set_menu( self._make_axis_menu() )
-            self.omenu.show_all()
-        else:
-            self.omenu.hide()
-
-        self.set_active(range(len(self._axes)))
-
-
     def _make_axis_menu(self):
         # called by self._update*()
 
@@ -952,9 +901,15 @@ class NavigationToolbar(gtk.Toolbar):
         self.canvas.draw()
         return True
 
+    def get_filechooser(self):
+        return FileChooserDialog(
+            title='Save the figure',
+            parent=self.win,
+            filetypes=self.canvas.get_supported_filetypes(),
+            default_filetype=self.canvas.get_default_filetype())
 
     def save_figure(self, button):
-        fname, format = self.fileselect.get_filename_from_user()
+        fname, format = self.get_filechooser().get_filename_from_user()
         if fname:
             try:
                 self.canvas.print_figure(fname, format=format)
@@ -962,111 +917,80 @@ class NavigationToolbar(gtk.Toolbar):
                 error_msg_gtk(str(e), parent=self)
 
 
-if gtk.pygtk_version >= (2,4,0):
-    class FileChooserDialog(gtk.FileChooserDialog):
-        """GTK+ 2.4 file selector which remembers the last file/directory
-        selected and presents the user with a menu of supported image formats
-        """
-        def __init__ (self,
-                      title   = 'Save file',
-                      parent  = None,
-                      action  = gtk.FILE_CHOOSER_ACTION_SAVE,
-                      buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                                 gtk.STOCK_SAVE,   gtk.RESPONSE_OK),
-                      path    = None,
-                      filetypes = [],
-                      default_filetype = None
-                      ):
-            super (FileChooserDialog, self).__init__ (title, parent, action,
-                                                      buttons)
-            self.set_default_response (gtk.RESPONSE_OK)
+class FileChooserDialog(gtk.FileChooserDialog):
+    """GTK+ 2.4 file selector which remembers the last file/directory
+    selected and presents the user with a menu of supported image formats
+    """
+    def __init__ (self,
+                  title   = 'Save file',
+                  parent  = None,
+                  action  = gtk.FILE_CHOOSER_ACTION_SAVE,
+                  buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+                             gtk.STOCK_SAVE,   gtk.RESPONSE_OK),
+                  path    = None,
+                  filetypes = [],
+                  default_filetype = None
+                  ):
+        super (FileChooserDialog, self).__init__ (title, parent, action,
+                                                  buttons)
+        self.set_default_response (gtk.RESPONSE_OK)
 
-            if not path: path = os.getcwd() + os.sep
+        if not path: path = os.getcwd() + os.sep
 
-            # create an extra widget to list supported image formats
-            self.set_current_folder (path)
-            self.set_current_name ('image.' + default_filetype)
+        # create an extra widget to list supported image formats
+        self.set_current_folder (path)
+        self.set_current_name ('image.' + default_filetype)
 
-            hbox = gtk.HBox (spacing=10)
-            hbox.pack_start (gtk.Label ("File Format:"), expand=False)
+        hbox = gtk.HBox (spacing=10)
+        hbox.pack_start (gtk.Label ("File Format:"), expand=False)
 
-            liststore = gtk.ListStore(gobject.TYPE_STRING)
-            cbox = gtk.ComboBox(liststore)
-            cell = gtk.CellRendererText()
-            cbox.pack_start(cell, True)
-            cbox.add_attribute(cell, 'text', 0)
-            hbox.pack_start (cbox)
+        liststore = gtk.ListStore(gobject.TYPE_STRING)
+        cbox = gtk.ComboBox(liststore)
+        cell = gtk.CellRendererText()
+        cbox.pack_start(cell, True)
+        cbox.add_attribute(cell, 'text', 0)
+        hbox.pack_start (cbox)
 
-            self.filetypes = filetypes
-            self.sorted_filetypes = filetypes.items()
-            self.sorted_filetypes.sort()
-            default = 0
-            for i, (ext, name) in enumerate(self.sorted_filetypes):
-                cbox.append_text ("%s (*.%s)" % (name, ext))
-                if ext == default_filetype:
-                    default = i
-            cbox.set_active(default)
-            self.ext = default_filetype
+        self.filetypes = filetypes
+        self.sorted_filetypes = filetypes.items()
+        self.sorted_filetypes.sort()
+        default = 0
+        for i, (ext, name) in enumerate(self.sorted_filetypes):
+            cbox.append_text ("%s (*.%s)" % (name, ext))
+            if ext == default_filetype:
+                default = i
+        cbox.set_active(default)
+        self.ext = default_filetype
 
-            def cb_cbox_changed (cbox, data=None):
-                """File extension changed"""
-                head, filename = os.path.split(self.get_filename())
-                root, ext = os.path.splitext(filename)
-                ext = ext[1:]
-                new_ext = self.sorted_filetypes[cbox.get_active()][0]
-                self.ext = new_ext
+        def cb_cbox_changed (cbox, data=None):
+            """File extension changed"""
+            head, filename = os.path.split(self.get_filename())
+            root, ext = os.path.splitext(filename)
+            ext = ext[1:]
+            new_ext = self.sorted_filetypes[cbox.get_active()][0]
+            self.ext = new_ext
 
-                if ext in self.filetypes:
-                    filename = root + '.' + new_ext
-                elif ext == '':
-                    filename = filename.rstrip('.') + '.' + new_ext
+            if ext in self.filetypes:
+                filename = root + '.' + new_ext
+            elif ext == '':
+                filename = filename.rstrip('.') + '.' + new_ext
 
-                self.set_current_name (filename)
-            cbox.connect ("changed", cb_cbox_changed)
+            self.set_current_name (filename)
+        cbox.connect ("changed", cb_cbox_changed)
 
-            hbox.show_all()
-            self.set_extra_widget(hbox)
-
-        def get_filename_from_user (self):
-            while True:
-                filename = None
-                if self.run() != int(gtk.RESPONSE_OK):
-                    break
-                filename = self.get_filename()
-                break
-
-            self.hide()
-            return filename, self.ext
-else:
-    class FileSelection(gtk.FileSelection):
-        """GTK+ 2.2 and lower file selector which remembers the last
-        file/directory selected
-        """
-        def __init__(self, path=None, title='Select a file', parent=None):
-            super(FileSelection, self).__init__(title)
-
-            if path: self.path = path
-            else:    self.path = os.getcwd() + os.sep
-
-            if parent: self.set_transient_for(parent)
-
-        def get_filename_from_user(self, path=None, title=None):
-            if path:  self.path = path
-            if title: self.set_title(title)
-            self.set_filename(self.path)
+        hbox.show_all()
+        self.set_extra_widget(hbox)
 
+    def get_filename_from_user (self):
+        while True:
             filename = None
-            if self.run() == int(gtk.RESPONSE_OK):
-                self.path = filename = self.get_filename()
-            self.hide()
-
-            ext = None
-            if filename is not None:
-                ext = os.path.splitext(filename)[1]
-                if ext.startswith('.'):
-                    ext = ext[1:]
-            return filename, ext
+            if self.run() != int(gtk.RESPONSE_OK):
+                break
+            filename = self.get_filename()
+            break
 
+        self.hide()
+        return filename, self.ext
 
 class DialogLineprops:
     """
@@ -1235,13 +1159,13 @@ class DialogLineprops:
 # versions of pygtk, so we have to use a PNG file instead.
 try:
 
-    if gtk.pygtk_version < (2, 8, 0):
+    if gtk.pygtk_version < (2, 8, 0) or sys.platform == 'win32':
         icon_filename = 'matplotlib.png'
     else:
         icon_filename = 'matplotlib.svg'
-    gtk.window_set_default_icon_from_file (
-        os.path.join (matplotlib.rcParams['datapath'], 'images', icon_filename))
+    window_icon = os.path.join(matplotlib.rcParams['datapath'], 'images', icon_filename)
 except:
+    window_icon = None
     verbose.report('Could not load matplotlib icon: %s' % sys.exc_info()[1])
 
 
