@@ -7,13 +7,16 @@ import matplotlib
 from matplotlib import verbose
 from matplotlib.cbook import is_string_like, onetrue
 from matplotlib.backend_bases import RendererBase, GraphicsContextBase, \
-     FigureManagerBase, FigureCanvasBase, NavigationToolbar2, cursors
+     FigureManagerBase, FigureCanvasBase, NavigationToolbar2, IdleEvent, cursors
 from matplotlib._pylab_helpers import Gcf
 from matplotlib.figure import Figure
 from matplotlib.mathtext import MathTextParser
 from matplotlib.widgets import SubplotTool
 
-from PyQt4 import QtCore, QtGui, Qt
+try:
+    from PyQt4 import QtCore, QtGui, Qt
+except ImportError:
+    raise ImportError("Qt4 backend requires that PyQt4 is installed.")
 
 backend_version = "0.9.1"
 def fn_name(): return sys._getframe(1).f_code.co_name
@@ -34,7 +37,7 @@ def draw_if_interactive():
     if matplotlib.is_interactive():
         figManager =  Gcf.get_active()
         if figManager != None:
-            figManager.canvas.draw()
+            figManager.canvas.draw_idle()
 
 def _create_qApp():
     """
@@ -94,10 +97,22 @@ class FigureCanvasQT( QtGui.QWidget, FigureCanvasBase ):
         FigureCanvasBase.__init__( self, figure )
         self.figure = figure
         self.setMouseTracking( True )
-
+        self._idle = True
+        # hide until we can test and fix
+        #self.startTimer(backend_IdleEvent.milliseconds)
         w,h = self.get_width_height()
         self.resize( w, h )
 
+    def __timerEvent(self, event):
+        # hide until we can test and fix
+        self.mpl_idle_event(event)
+
+    def enterEvent(self, event):
+        FigureCanvasBase.enter_notify_event(self, event)
+
+    def leaveEvent(self, event):
+        FigureCanvasBase.leave_notify_event(self, event)
+
     def mousePressEvent( self, event ):
         x = event.pos().x()
         # flipy so y=0 is bottom of canvas
@@ -121,6 +136,16 @@ class FigureCanvasQT( QtGui.QWidget, FigureCanvasBase ):
         FigureCanvasBase.button_release_event( self, x, y, button )
         if DEBUG: print 'button released'
 
+    def wheelEvent( self, event ):
+        x = event.x()
+        # flipy so y=0 is bottom of canvas
+        y = self.figure.bbox.height - event.y()
+        # from QWheelEvent::delta doc
+        steps = event.delta()/120
+        if (event.orientation() == Qt.Qt.Vertical):
+            FigureCanvasBase.scroll_event( self, x, y, steps)
+            if DEBUG: print 'scroll event : delta = %i, steps = %i ' % (event.delta(),steps)
+
     def keyPressEvent( self, event ):
         key = self._get_key( event )
         FigureCanvasBase.key_press_event( self, key )
@@ -133,7 +158,6 @@ class FigureCanvasQT( QtGui.QWidget, FigureCanvasBase ):
 
     def resizeEvent( self, event ):
         if DEBUG: print 'resize (%d x %d)' % (event.size().width(), event.size().height())
-        QtGui.QWidget.resizeEvent( self, event )
         w = event.size().width()
         h = event.size().height()
         if DEBUG: print "FigureCanvasQtAgg.resizeEvent(", w, ",", h, ")"
@@ -142,19 +166,8 @@ class FigureCanvasQT( QtGui.QWidget, FigureCanvasBase ):
         hinch = h/dpival
         self.figure.set_size_inches( winch, hinch )
         self.draw()
-
-    def resize( self, w, h ):
-        # Pass through to Qt to resize the widget.
-        QtGui.QWidget.resize( self, w, h )
-
-        # Resize the figure by converting pixels to inches.
-        pixelPerInch = self.figure.dpi
-        wInch = w / pixelPerInch
-        hInch = h / pixelPerInch
-        self.figure.set_size_inches( wInch, hInch )
-
-        # Redraw everything.
-        self.draw()
+        self.update()
+        QtGui.QWidget.resizeEvent(self, event)
 
     def sizeHint( self ):
         w, h = self.get_width_height()
@@ -166,7 +179,7 @@ class FigureCanvasQT( QtGui.QWidget, FigureCanvasBase ):
     def _get_key( self, event ):
         if event.key() < 256:
             key = str(event.text())
-        elif self.keyvald.has_key( event.key() ):
+        elif event.key() in self.keyvald:
             key = self.keyvald[ event.key() ]
         else:
             key = None
@@ -176,6 +189,23 @@ class FigureCanvasQT( QtGui.QWidget, FigureCanvasBase ):
     def flush_events(self):
         Qt.qApp.processEvents()
 
+    def start_event_loop(self,timeout):
+        FigureCanvasBase.start_event_loop_default(self,timeout)
+    start_event_loop.__doc__=FigureCanvasBase.start_event_loop_default.__doc__
+
+    def stop_event_loop(self):
+        FigureCanvasBase.stop_event_loop_default(self)
+    stop_event_loop.__doc__=FigureCanvasBase.stop_event_loop_default.__doc__
+
+    def draw_idle(self):
+        'update drawing area only if idle'
+        d = self._idle
+        self._idle = False
+        def idle_draw(*args):
+            self.draw()
+            self._idle = True
+        if d: QtCore.QTimer.singleShot(0, idle_draw)
+
 class FigureManagerQT( FigureManagerBase ):
     """
     Public attributes
@@ -306,13 +336,6 @@ class NavigationToolbar2QT( NavigationToolbar2, QtGui.QToolBar ):
         # reference holder for subplots_adjust window
         self.adj_window = None
 
-    def destroy( self ):
-        for text, tooltip_text, image_file, callback in self.toolitems:
-            if text is not None:
-                QtCore.QObject.disconnect( self.buttons[ text ],
-                                           QtCore.SIGNAL( 'clicked()' ),
-                                           getattr( self, callback ) )
-
     def dynamic_update( self ):
         self.canvas.draw()
 
