@@ -61,9 +61,6 @@ class FigureCanvasQTAgg( FigureCanvasQT, FigureCanvasAgg ):
         self.replot = True
         self.setAttribute(QtCore.Qt.WA_OpaquePaintEvent)
 
-    def resizeEvent( self, e ):
-        FigureCanvasQT.resizeEvent( self, e )
-
     def drawRectangle( self, rect ):
         self.rect = rect
         self.drawRect = True
