@@ -513,7 +513,8 @@ in the normalized axes coordinate.
                 color = handle.get_colors()[0]
                 legline.set_color(color)
                 legline.set_linewidth(lw)
-                legline.set_dashes(dashes)
+                if dashes[0] is not None: # dashed line
+                    legline.set_dashes(dashes[1])
                 handle_list.append(legline)
 
             elif isinstance(handle, RegularPolyCollection):
@@ -587,8 +588,9 @@ in the normalized axes coordinate.
         # We calculate number of lows in each column. The first
         # (num_largecol) columns will have (nrows+1) rows, and remaing
         # (num_smallcol) columns will have (nrows) rows.
-        nrows, num_largecol = divmod(len(handleboxes), self._ncol)
-        num_smallcol = self._ncol-num_largecol
+        ncol = min(self._ncol, len(handleboxes))
+        nrows, num_largecol = divmod(len(handleboxes), ncol)
+        num_smallcol = ncol-num_largecol
 
         # starting index of each column and number of rows in it.
         largecol = safezip(range(0, num_largecol*(nrows+1), (nrows+1)),
@@ -849,3 +851,4 @@ in the normalized axes coordinate.
 
         return ox, oy
 
+
