@@ -10,13 +10,15 @@ from numpy import ma
 
 from matplotlib._path import point_in_path, get_path_extents, \
     point_in_path_collection, get_path_collection_extents, \
-    path_in_path, path_intersects_path, convert_path_to_polygons
-from matplotlib.cbook import simple_linear_interpolation
+    path_in_path, path_intersects_path, convert_path_to_polygons, \
+    cleanup_path
+from matplotlib.cbook import simple_linear_interpolation, maxdict
+from matplotlib import rcParams
 
 class Path(object):
     """
-    Path represents a series of possibly disconnected, possibly
-    closed, line and curve segments.
+    :class:`Path` represents a series of possibly disconnected,
+    possibly closed, line and curve segments.
 
     The underlying storage is made up of two parallel numpy arrays:
       - *vertices*: an Nx2 float array of vertices
@@ -56,21 +58,29 @@ class Path(object):
     :class:`Path` objects, as an optimization, do not store a *codes*
     at all, but have a default one provided for them by
     :meth:`iter_segments`.
+
+    Note also that the vertices and codes arrays should be treated as
+    immutable -- there are a number of optimizations and assumptions
+    made up front in the constructor that will not change when the
+    data changes.
     """
 
     # Path codes
-    STOP      = 0 # 1 vertex
-    MOVETO    = 1 # 1 vertex
-    LINETO    = 2 # 1 vertex
-    CURVE3    = 3 # 2 vertices
-    CURVE4    = 4 # 3 vertices
-    CLOSEPOLY = 5 # 1 vertex
+    STOP      = 0    # 1 vertex
+    MOVETO    = 1    # 1 vertex
+    LINETO    = 2    # 1 vertex
+    CURVE3    = 3    # 2 vertices
+    CURVE4    = 4    # 3 vertices
+    CLOSEPOLY = 0x4f # 1 vertex
 
-    NUM_VERTICES = [1, 1, 1, 2, 3, 1]
+    NUM_VERTICES = [1, 1, 1, 2,
+                    3, 1, 1, 1,
+                    1, 1, 1, 1,
+                    1, 1, 1, 1]
 
     code_type = np.uint8
 
-    def __init__(self, vertices, codes=None):
+    def __init__(self, vertices, codes=None, _interpolation_steps=1):
         """
         Create a new path with the given vertices and codes.
 
@@ -84,51 +94,70 @@ class Path(object):
         dimension.
 
         If *codes* is None, *vertices* will be treated as a series of
-        line segments.  If *vertices* contains masked values, the
-        resulting path will be compressed, with ``MOVETO`` codes
-        inserted in the correct places to jump over the masked
-        regions.
+        line segments.
+
+        If *vertices* contains masked values, they will be converted
+        to NaNs which are then handled correctly by the Agg
+        PathIterator and other consumers of path data, such as
+        :meth:`iter_segments`.
+
+        *interpolation_steps* is used as a hint to certain projections,
+        such as Polar, that this path should be linearly interpolated
+        immediately before drawing.  This attribute is primarily an
+        implementation detail and is not intended for public use.
         """
         if ma.isMaskedArray(vertices):
-            is_mask = True
-            mask = ma.getmask(vertices)
+            vertices = vertices.astype(np.float_).filled(np.nan)
         else:
-            is_mask = False
             vertices = np.asarray(vertices, np.float_)
-            mask = ma.nomask
 
         if codes is not None:
             codes = np.asarray(codes, self.code_type)
             assert codes.ndim == 1
             assert len(codes) == len(vertices)
 
-        # The path being passed in may have masked values.  However,
-        # the backends (and any affine transformations in matplotlib
-        # itself), are not expected to deal with masked arrays, so we
-        # must remove them from the array (using compressed), and add
-        # MOVETO commands to the codes array accordingly.
-        if is_mask:
-            if mask is not ma.nomask:
-                mask1d = np.logical_or.reduce(mask, axis=1)
-                gmask1d = np.invert(mask1d)
-                if codes is None:
-                    codes = np.empty((len(vertices)), self.code_type)
-                    codes.fill(self.LINETO)
-                    codes[0] = self.MOVETO
-                vertices = vertices[gmask1d].filled() # ndarray
-                codes[np.roll(mask1d, 1)] = self.MOVETO
-                codes = codes[gmask1d] # np.compress is much slower
-            else:
-                vertices = np.asarray(vertices, np.float_)
-
         assert vertices.ndim == 2
         assert vertices.shape[1] == 2
 
+        self.should_simplify = (rcParams['path.simplify'] and
+                                (len(vertices) >= 128 and
+                                 (codes is None or np.all(codes <= Path.LINETO))))
+        self.simplify_threshold = rcParams['path.simplify_threshold']
+        self.has_nonfinite = not np.isfinite(vertices).all()
         self.codes = codes
         self.vertices = vertices
+        self._interpolation_steps = _interpolation_steps
 
-    #@staticmethod
-    def make_compound_path(*args):
+    @classmethod
+    def make_compound_path_from_polys(cls, XY):
+        """
+        (static method) Make a compound path object to draw a number
+        of polygons with equal numbers of sides XY is a (numpolys x
+        numsides x 2) numpy array of vertices.  Return object is a
+        :class:`Path`
+
+        .. plot:: mpl_examples/api/histogram_path_demo.py
+
+        """
+
+        # for each poly: 1 for the MOVETO, (numsides-1) for the LINETO, 1 for the
+        # CLOSEPOLY; the vert for the closepoly is ignored but we still need
+        # it to keep the codes aligned with the vertices
+        numpolys, numsides, two = XY.shape
+        assert(two==2)
+        stride = numsides + 1
+        nverts = numpolys * stride
+        verts = np.zeros((nverts, 2))
+        codes = np.ones(nverts, int) * cls.LINETO
+        codes[0::stride] = cls.MOVETO
+        codes[numsides::stride] = cls.CLOSEPOLY
+        for i in range(numsides):
+            verts[i::stride] = XY[:,i]
+
+        return cls(verts, codes)
+
+    @classmethod
+    def make_compound_path(cls, *args):
         """
         (staticmethod) Make a compound path from a list of Path
         objects.  Only polygons (not curves) are supported.
@@ -142,14 +171,13 @@ class Path(object):
         vertices = np.vstack([x.vertices for x in args])
         vertices.reshape((total_length, 2))
 
-        codes = Path.LINETO * np.ones(total_length)
+        codes = cls.LINETO * np.ones(total_length)
         i = 0
         for length in lengths:
-            codes[i] = Path.MOVETO
+            codes[i] = cls.MOVETO
             i += length
 
-        return Path(vertices, codes)
-    make_compound_path = staticmethod(make_compound_path)
+        return cls(vertices, codes)
 
     def __repr__(self):
         return "Path(%s, %s)" % (self.vertices, self.codes)
@@ -157,21 +185,43 @@ class Path(object):
     def __len__(self):
         return len(self.vertices)
 
-    def iter_segments(self):
+    def iter_segments(self, transform=None, remove_nans=True, clip=None,
+                      quantize=False, simplify=None, curves=True):
         """
         Iterates over all of the curve segments in the path.  Each
         iteration returns a 2-tuple (*vertices*, *code*), where
         *vertices* is a sequence of 1 - 3 coordinate pairs, and *code* is
         one of the :class:`Path` codes.
+
+        Additionally, this method can provide a number of standard
+        cleanups and conversions to the path.
+
+        *transform*: if not None, the given affine transformation will
+         be applied to the path.
+
+        *remove_nans*: if True, will remove all NaNs from the path and
+         insert MOVETO commands to skip over them.
+
+        *clip*: if not None, must be a four-tuple (x1, y1, x2, y2)
+         defining a rectangle in which to clip the path.
+
+        *quantize*: if None, auto-quantize.  If True, force quantize,
+         and if False, don't quantize.
+
+        *simplify*: if True, perform simplification, to remove
+         vertices that do not affect the appearance of the path.  If
+         False, perform no simplification.  If None, use the
+         should_simplify member variable.
+
+        *curves*: If True, curve segments will be returned as curve
+         segments.  If False, all curves will be converted to line
+         segments.
         """
         vertices = self.vertices
         if not len(vertices):
             return
 
         codes        = self.codes
-        len_vertices = len(vertices)
-        isnan        = np.isnan
-        any          = np.any
 
         NUM_VERTICES = self.NUM_VERTICES
         MOVETO       = self.MOVETO
@@ -179,45 +229,34 @@ class Path(object):
         CLOSEPOLY    = self.CLOSEPOLY
         STOP         = self.STOP
 
-        if codes is None:
-            next_code = MOVETO
-            for v in vertices:
-                if any(isnan(v)):
-                    next_code = MOVETO
-                else:
-                    yield v, next_code
-                    next_code = LINETO
-        else:
-            i = 0
-            was_nan = False
-            while i < len_vertices:
-                code = codes[i]
-                if code == CLOSEPOLY:
-                    yield [], code
-                    i += 1
-                elif code == STOP:
-                    return
-                else:
-                    num_vertices = NUM_VERTICES[int(code)]
-                    curr_vertices = vertices[i:i+num_vertices].flatten()
-                    if any(isnan(curr_vertices)):
-                        was_nan = True
-                    elif was_nan:
-                        yield curr_vertices[-2:], MOVETO
-                        was_nan = False
-                    else:
-                        yield curr_vertices, code
-                    i += num_vertices
+        vertices, codes = cleanup_path(self, transform, remove_nans, clip,
+                                       quantize, simplify, curves)
+        len_vertices = len(vertices)
+
+        i = 0
+        while i < len_vertices:
+            code = codes[i]
+            if code == STOP:
+                return
+            else:
+                num_vertices = NUM_VERTICES[int(code) & 0xf]
+                curr_vertices = vertices[i:i+num_vertices].flatten()
+                yield curr_vertices, code
+                i += num_vertices
 
     def transformed(self, transform):
         """
         Return a transformed copy of the path.
 
-        See :class:`matplotlib.transforms.TransformedPath` for a path
-        that will cache the transformed result and automatically
-        update when the transform changes.
+        .. seealso::
+
+            :class:`matplotlib.transforms.TransformedPath`
+                A specialized path class that will cache the
+                transformed result and automatically update when the
+                transform changes.
         """
-        return Path(transform.transform(self.vertices), self.codes)
+        return Path(transform.transform(self.vertices), self.codes,
+                    self._interpolation_steps)
 
     def contains_point(self, point, transform=None):
         """
@@ -255,21 +294,29 @@ class Path(object):
             transform = transform.frozen()
         return Bbox(get_path_extents(self, transform))
 
-    def intersects_path(self, other):
+    def intersects_path(self, other, filled=True):
         """
         Returns *True* if this path intersects another given path.
-        """
-        return path_intersects_path(self, other)
 
-    def intersects_bbox(self, bbox):
+        *filled*, when True, treats the paths as if they were filled.
+        That is, if one path completely encloses the other,
+        :meth:`intersects_path` will return True.
+        """
+        return path_intersects_path(self, other, filled)
+
+    def intersects_bbox(self, bbox, filled=True):
         """
         Returns *True* if this path intersects a given
         :class:`~matplotlib.transforms.Bbox`.
+
+        *filled*, when True, treats the path as if it was filled.
+        That is, if one path completely encloses the other,
+        :meth:`intersects_path` will return True.
         """
         from transforms import BboxTransformTo
         rectangle = self.unit_rectangle().transformed(
             BboxTransformTo(bbox))
-        result = self.intersects_path(rectangle)
+        result = self.intersects_path(rectangle, filled)
         return result
 
     def interpolated(self, steps):
@@ -277,6 +324,9 @@ class Path(object):
         Returns a new path resampled to length N x steps.  Does not
         currently handle interpolating curves.
         """
+        if steps == 1:
+            return self
+
         vertices = simple_linear_interpolation(self.vertices, steps)
         codes = self.codes
         if codes is not None:
@@ -315,7 +365,7 @@ class Path(object):
         return convert_path_to_polygons(self, transform, width, height)
 
     _unit_rectangle = None
-    #@classmethod
+    @classmethod
     def unit_rectangle(cls):
         """
         (staticmethod) Returns a :class:`Path` of the unit rectangle
@@ -323,12 +373,12 @@ class Path(object):
         """
         if cls._unit_rectangle is None:
             cls._unit_rectangle = \
-                Path([[0.0, 0.0], [1.0, 0.0], [1.0, 1.0], [0.0, 1.0], [0.0, 0.0]])
+                cls([[0.0, 0.0], [1.0, 0.0], [1.0, 1.0], [0.0, 1.0], [0.0, 0.0]])
         return cls._unit_rectangle
-    unit_rectangle = classmethod(unit_rectangle)
 
     _unit_regular_polygons = WeakValueDictionary()
-    #@classmethod
+
+    @classmethod
     def unit_regular_polygon(cls, numVertices):
         """
         (staticmethod) Returns a :class:`Path` for a unit regular
@@ -346,13 +396,13 @@ class Path(object):
             # "points-up"
             theta += np.pi / 2.0
             verts = np.concatenate((np.cos(theta), np.sin(theta)), 1)
-            path = Path(verts)
+            path = cls(verts)
             cls._unit_regular_polygons[numVertices] = path
         return path
-    unit_regular_polygon = classmethod(unit_regular_polygon)
 
     _unit_regular_stars = WeakValueDictionary()
-    #@classmethod
+
+    @classmethod
     def unit_regular_star(cls, numVertices, innerCircle=0.5):
         """
         (staticmethod) Returns a :class:`Path` for a unit regular star
@@ -372,12 +422,11 @@ class Path(object):
             r = np.ones(ns2 + 1)
             r[1::2] = innerCircle
             verts = np.vstack((r*np.cos(theta), r*np.sin(theta))).transpose()
-            path = Path(verts)
+            path = cls(verts)
             cls._unit_regular_polygons[(numVertices, innerCircle)] = path
         return path
-    unit_regular_star = classmethod(unit_regular_star)
 
-    #@classmethod
+    @classmethod
     def unit_regular_asterisk(cls, numVertices):
         """
         (staticmethod) Returns a :class:`Path` for a unit regular
@@ -385,10 +434,10 @@ class Path(object):
         centered at (0, 0).
         """
         return cls.unit_regular_star(numVertices, 0.0)
-    unit_regular_asterisk = classmethod(unit_regular_asterisk)
 
     _unit_circle = None
-    #@classmethod
+
+    @classmethod
     def unit_circle(cls):
         """
         (staticmethod) Returns a :class:`Path` of the unit circle.
@@ -446,11 +495,10 @@ class Path(object):
             codes[0] = cls.MOVETO
             codes[-1] = cls.CLOSEPOLY
 
-            cls._unit_circle = Path(vertices, codes)
+            cls._unit_circle = cls(vertices, codes)
         return cls._unit_circle
-    unit_circle = classmethod(unit_circle)
 
-    #@classmethod
+    @classmethod
     def arc(cls, theta1, theta2, n=None, is_wedge=False):
         """
         (staticmethod) Returns an arc on the unit circle from angle
@@ -503,19 +551,19 @@ class Path(object):
 
         if is_wedge:
             length = n * 3 + 4
-            vertices = np.zeros((length, 2), np.float_)
-            codes = Path.CURVE4 * np.ones((length, ), Path.code_type)
+            vertices = np.empty((length, 2), np.float_)
+            codes = cls.CURVE4 * np.ones((length, ), cls.code_type)
             vertices[1] = [xA[0], yA[0]]
-            codes[0:2] = [Path.MOVETO, Path.LINETO]
-            codes[-2:] = [Path.LINETO, Path.CLOSEPOLY]
+            codes[0:2] = [cls.MOVETO, cls.LINETO]
+            codes[-2:] = [cls.LINETO, cls.CLOSEPOLY]
             vertex_offset = 2
             end = length - 2
         else:
             length = n * 3 + 1
-            vertices = np.zeros((length, 2), np.float_)
-            codes = Path.CURVE4 * np.ones((length, ), Path.code_type)
+            vertices = np.empty((length, 2), np.float_)
+            codes = cls.CURVE4 * np.ones((length, ), cls.code_type)
             vertices[0] = [xA[0], yA[0]]
-            codes[0] = Path.MOVETO
+            codes[0] = cls.MOVETO
             vertex_offset = 1
             end = length
 
@@ -526,10 +574,9 @@ class Path(object):
         vertices[vertex_offset+2:end:3, 0] = xB
         vertices[vertex_offset+2:end:3, 1] = yB
 
-        return Path(vertices, codes)
-    arc = classmethod(arc)
+        return cls(vertices, codes)
 
-    #@classmethod
+    @classmethod
     def wedge(cls, theta1, theta2, n=None):
         """
         (staticmethod) Returns a wedge of the unit circle from angle
@@ -540,7 +587,28 @@ class Path(object):
         determined based on the delta between *theta1* and *theta2*.
         """
         return cls.arc(theta1, theta2, n, True)
-    wedge = classmethod(wedge)
+
+    _hatch_dict = maxdict(8)
+
+    @classmethod
+    def hatch(cls, hatchpattern, density=6):
+        """
+        Given a hatch specifier, *hatchpattern*, generates a Path that
+        can be used in a repeated hatching pattern.  *density* is the
+        number of lines per unit square.
+        """
+        from matplotlib.hatch import get_path
+
+        if hatchpattern is None:
+            return None
+
+        hatch_path = cls._hatch_dict.get((hatchpattern, density))
+        if hatch_path is not None:
+            return hatch_path
+
+        hatch_path = get_path(hatchpattern, density)
+        cls._hatch_dict[(hatchpattern, density)] = hatch_path
+        return hatch_path
 
 _get_path_collection_extents = get_path_collection_extents
 def get_path_collection_extents(*args):
