@@ -324,6 +324,33 @@ class QuiverKey(martist.Artist):
     quiverkey_doc = _quiverkey_doc
 
 
+# This is a helper function that parses out the various combination of
+# arguments for doing colored vector plots.  Pulling it out here
+# allows both Quiver and Barbs to use it
+def _parse_args(*args):
+    X, Y, U, V, C = [None]*5
+    args = list(args)
+
+    # The use of atleast_1d allows for handling scalar arguments while also
+    # keeping masked arrays
+    if len(args) == 3 or len(args) == 5:
+        C = np.atleast_1d(args.pop(-1))
+    V = np.atleast_1d(args.pop(-1))
+    U = np.atleast_1d(args.pop(-1))
+    if U.ndim == 1:
+        nr, nc = 1, U.shape[0]
+    else:
+        nr, nc = U.shape
+    if len(args) == 2: # remaining after removing U,V,C
+        X, Y = [np.array(a).ravel() for a in args]
+        if len(X) == nc and len(Y) == nr:
+            X, Y = [a.ravel() for a in np.meshgrid(X, Y)]
+    else:
+        indexgrid = np.meshgrid(np.arange(nc), np.arange(nr))
+        X, Y = [np.ravel(a) for a in indexgrid]
+    return X, Y, U, V, C
+
+
 class Quiver(collections.PolyCollection):
     """
     Specialized PolyCollection for arrows.
@@ -343,7 +370,7 @@ class Quiver(collections.PolyCollection):
     """
     def __init__(self, ax, *args, **kw):
         self.ax = ax
-        X, Y, U, V, C = self._parse_args(*args)
+        X, Y, U, V, C = _parse_args(*args)
         self.X = X
         self.Y = Y
         self.XY = np.hstack((X[:,np.newaxis], Y[:,np.newaxis]))
@@ -388,26 +415,6 @@ class Quiver(collections.PolyCollection):
         by the following pylab interface documentation:
         %s""" % _quiver_doc
 
-    def _parse_args(self, *args):
-        X, Y, U, V, C = [None]*5
-        args = list(args)
-        if len(args) == 3 or len(args) == 5:
-            C = np.asanyarray(args.pop(-1))
-        V = np.asanyarray(args.pop(-1))
-        U = np.asanyarray(args.pop(-1))
-        if U.ndim == 1:
-            nr, nc = 1, U.shape[0]
-        else:
-            nr, nc = U.shape
-        if len(args) == 2: # remaining after removing U,V,C
-            X, Y = [np.array(a).ravel() for a in args]
-            if len(X) == nc and len(Y) == nr:
-                X, Y = [a.ravel() for a in np.meshgrid(X, Y)]
-        else:
-            indexgrid = np.meshgrid(np.arange(nc), np.arange(nr))
-            X, Y = [np.ravel(a) for a in indexgrid]
-        return X, Y, U, V, C
-
     def _init(self):
         """initialization delayed until first draw;
         allow time for axes setup.
@@ -489,7 +496,11 @@ class Quiver(collections.PolyCollection):
         a = np.absolute(uv)
         if self.scale is None:
             sn = max(10, math.sqrt(self.N))
-            scale = 1.8 * a[~self.Umask].mean() * sn / self.span # crude auto-scaling
+            if self.Umask is not ma.nomask:
+                amean = a[~self.Umask].mean()
+            else:
+                amean = a.mean()
+            scale = 1.8 * amean * sn / self.span # crude auto-scaling
             self.scale = scale
         length = a/(self.scale*self.width)
         X, Y = self._h_arrows(length)
@@ -499,6 +510,7 @@ class Quiver(collections.PolyCollection):
             theta = np.angle(uv)
         else:
             theta = ma.masked_invalid(self.angles, copy=False).filled(0)
+            theta = theta.ravel()
             theta *= (np.pi/180.0)
         theta.shape = (theta.shape[0], 1) # for broadcasting
         xy = (X+Y*1j) * np.exp(1j*theta)*self.width
@@ -752,7 +764,7 @@ class Barbs(collections.PolyCollection):
             kw['facecolors'] = flagcolor
 
         #Parse out the data arrays from the various configurations supported
-        x, y, u, v, c = self._parse_args(*args)
+        x, y, u, v, c = _parse_args(*args)
         self.x = x
         self.y = y
         xy = np.hstack((x[:,np.newaxis], y[:,np.newaxis]))
@@ -938,28 +950,6 @@ class Barbs(collections.PolyCollection):
 
         return barb_list
 
-    #Taken shamelessly from Quiver
-    def _parse_args(self, *args):
-        X, Y, U, V, C = [None]*5
-        args = list(args)
-        if len(args) == 3 or len(args) == 5:
-            C = ma.masked_invalid(args.pop(-1), copy=False).ravel()
-        V = ma.masked_invalid(args.pop(-1), copy=False)
-        U = ma.masked_invalid(args.pop(-1), copy=False)
-        nn = np.shape(U)
-        nc = nn[0]
-        nr = 1
-        if len(nn) > 1:
-            nr = nn[1]
-        if len(args) == 2: # remaining after removing U,V,C
-            X, Y = [np.array(a).ravel() for a in args]
-            if len(X) == nc and len(Y) == nr:
-                X, Y = [a.ravel() for a in np.meshgrid(X, Y)]
-        else:
-            indexgrid = np.meshgrid(np.arange(nc), np.arange(nr))
-            X, Y = [np.ravel(a) for a in indexgrid]
-        return X, Y, U, V, C
-
     def set_UVC(self, U, V, C=None):
         self.u = ma.masked_invalid(U, copy=False).ravel()
         self.v = ma.masked_invalid(V, copy=False).ravel()
