@@ -34,6 +34,7 @@ or include these two lines in your script::
 """
 
 import copy, glob, os, shutil, sys, warnings
+from subprocess import Popen, PIPE, STDOUT
 
 try:
     from hashlib import md5
@@ -45,6 +46,8 @@ import numpy as np
 import matplotlib as mpl
 from matplotlib import rcParams
 from matplotlib._png import read_png
+import matplotlib.dviread as dviread
+import re
 
 DEBUG = False
 
@@ -52,7 +55,9 @@ if sys.platform.startswith('win'): cmd_split = '&'
 else: cmd_split = ';'
 
 def dvipng_hack_alpha():
-    stdin, stdout = os.popen4('dvipng -version')
+    p = Popen('dvipng -version', shell=True, stdin=PIPE, stdout=PIPE,
+        stderr=STDOUT, close_fds=(sys.platform!='win32'))
+    stdin, stdout = p.stdin, p.stdout
     for line in stdout:
         if line.startswith('dvipng '):
             version = line.split()[-1]
@@ -262,12 +267,83 @@ WARNING: found a TeX cache dir in the deprecated location "%s".
 
         return texfile
 
+
+    _re_vbox = re.compile(r"MatplotlibBox:\(([\d.]+)pt\+([\d.]+)pt\)x([\d.]+)pt")
+
+    def make_tex_preview(self, tex, fontsize):
+        """
+        Generate a tex file to render the tex string at a specific
+        font size.  It uses the preview.sty to determin the dimension
+        (width, height, descent) of the output.
+
+        returns the file name
+        """
+        basefile = self.get_basefile(tex, fontsize)
+        texfile = '%s.tex'%basefile
+        fh = file(texfile, 'w')
+        custom_preamble = self.get_custom_preamble()
+        fontcmd = {'sans-serif' : r'{\sffamily %s}',
+                   'monospace'  : r'{\ttfamily %s}'}.get(self.font_family,
+                                                         r'{\rmfamily %s}')
+        tex = fontcmd % tex
+
+        if rcParams['text.latex.unicode']:
+            unicode_preamble = """\usepackage{ucs}
+\usepackage[utf8x]{inputenc}"""
+        else:
+            unicode_preamble = ''
+
+
+
+        # newbox, setbox, immediate, etc. are used to find the box
+        # extent of the rendered text.
+
+
+        s = r"""\documentclass{article}
+%s
+%s
+%s
+\usepackage[active,showbox,tightpage]{preview}
+%%\usepackage[papersize={72in,72in}, body={70in,70in}, margin={1in,1in}]{geometry}
+
+%% we override the default showbox as it is treated as an error and makes
+%% the exit status not zero
+\def\showbox#1{\immediate\write16{MatplotlibBox:(\the\ht#1+\the\dp#1)x\the\wd#1}}
+
+\begin{document}
+\begin{preview}
+{\fontsize{%f}{%f}%s}
+\end{preview}
+\end{document}
+""" % (self._font_preamble, unicode_preamble, custom_preamble,
+       fontsize, fontsize*1.25, tex)
+        if rcParams['text.latex.unicode']:
+            fh.write(s.encode('utf8'))
+        else:
+            try:
+                fh.write(s)
+            except UnicodeEncodeError, err:
+                mpl.verbose.report("You are using unicode and latex, but have "
+                            "not enabled the matplotlib 'text.latex.unicode' "
+                            "rcParam.", 'helpful')
+                raise
+
+        fh.close()
+
+        return texfile
+
+
     def make_dvi(self, tex, fontsize):
         """
         generates a dvi file containing latex's layout of tex string
 
         returns the file name
         """
+
+
+        if rcParams['text.latex.preview']:
+            return self.make_dvi_preview(tex, fontsize)
+
         basefile = self.get_basefile(tex, fontsize)
         dvifile = '%s.dvi'% basefile
 
@@ -285,7 +361,12 @@ WARNING: found a TeX cache dir in the deprecated location "%s".
                 fh.close()
             except IOError:
                 report = 'No latex error report available.'
-            if exit_status:
+            try:
+                os.stat(dvifile)
+                exists = True
+            except OSError:
+                exists = False
+            if exit_status or not exists:
                 raise RuntimeError(('LaTeX was not able to process the following \
 string:\n%s\nHere is the full report generated by LaTeX: \n\n'% repr(tex)) + report)
             else: mpl.verbose.report(report, 'debug')
@@ -298,6 +379,55 @@ string:\n%s\nHere is the full report generated by LaTeX: \n\n'% repr(tex)) + rep
 
         return dvifile
 
+
+    def make_dvi_preview(self, tex, fontsize):
+        """
+        generates a dvi file containing latex's layout of tex
+        string. It calls make_tex_preview() method and store the size
+        information (width, height, descent) in a separte file.
+
+        returns the file name
+        """
+        basefile = self.get_basefile(tex, fontsize)
+        dvifile = '%s.dvi'% basefile
+        baselinefile = '%s.baseline'% basefile
+
+        if DEBUG or not os.path.exists(dvifile) or \
+               not os.path.exists(baselinefile):
+            texfile = self.make_tex_preview(tex, fontsize)
+            outfile = basefile+'.output'
+            command = self._get_shell_cmd('cd "%s"'% self.texcache,
+                            'latex -interaction=nonstopmode %s > "%s"'\
+                            %(os.path.split(texfile)[-1], outfile))
+            mpl.verbose.report(command, 'debug')
+            exit_status = os.system(command)
+            try:
+                fh = file(outfile)
+                report = fh.read()
+                fh.close()
+
+            except IOError:
+                report = 'No latex error report available.'
+            if exit_status:
+                raise RuntimeError(('LaTeX was not able to process the following \
+string:\n%s\nHere is the full report generated by LaTeX: \n\n'% repr(tex)) + report)
+            else: mpl.verbose.report(report, 'debug')
+
+            # find the box extent information in the latex output
+            # file and store them in ".baseline" file
+            m = TexManager._re_vbox.search(report)
+            open(basefile+'.baseline',"w").write(" ".join(m.groups()))
+
+            for fname in glob.glob(basefile+'*'):
+                if fname.endswith('dvi'): pass
+                elif fname.endswith('tex'): pass
+                elif fname.endswith('baseline'): pass
+                else:
+                    try: os.remove(fname)
+                    except OSError: pass
+
+        return dvifile
+
     def make_png(self, tex, fontsize, dpi):
         """
         generates a png file containing latex's rendering of tex string
@@ -441,3 +571,36 @@ process the flowing file:\n%s\nHere is the full report generated by dvipng: \
             self.rgba_arrayd[key] = Z
 
         return Z
+
+
+    def get_text_width_height_descent(self, tex, fontsize, renderer=None):
+        """
+        return width, heigth and descent of the text.
+        """
+
+        if renderer:
+            dpi_fraction = renderer.points_to_pixels(1.)
+        else:
+            dpi_fraction = 1.
+
+        if rcParams['text.latex.preview']:
+            # use preview.sty
+            basefile = self.get_basefile(tex, fontsize)
+            baselinefile = '%s.baseline'% basefile
+
+
+            if DEBUG or not os.path.exists(baselinefile):
+                dvifile = self.make_dvi_preview(tex, fontsize)
+
+            l = open(baselinefile).read().split()
+            height, depth, width = [float(l1)*dpi_fraction for l1 in l]
+            return width, height+depth, depth
+
+        else:
+            # use dviread. It sometimes returns a wrong descent.
+            dvifile = self.make_dvi(tex, fontsize)
+            dvi = dviread.Dvi(dvifile, 72*dpi_fraction)
+            page = iter(dvi).next()
+            dvi.close()
+            # A total height (including the descent) needs to be returned.
+            return page.width, page.height+page.descent, page.descent
