@@ -5,4 +5,4 @@ Poor man's setuptools script...
 from setuptools import setup
 execfile('setup.py',
          {'additional_params' :
-          {'namespace_packages' : ['matplotlib.toolkits']}})
+         {'namespace_packages' : ['mpl_toolkits']}})
